// fluid_project/CDrawBox.h

#ifndef CDRAWBOX_H_INCLUDED
#define CDRAWBOX_H_INCLUDED

#include <cassert>
#include <iostream>
#include <vector>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <FL/names.h>

class CPrimitiveLine {
    int x1;
    int y1;
    int x2;
    int y2;
    Fl_Color color;
public:
    CPrimitiveLine(
        int x1,
        int y1,
        int x2,
        int y2,
        Fl_Color color
        )
        : 
    x1( x1 ),
        y1( y1 ),
        x2( x2 ),
        y2( y2 ),
        color( color )
    {
    }

    void draw()
    {
        Fl_Color orig_color = fl_color();
        fl_color( color ); // default color is same as window background
        // note that line is drawn in window coordinates:
        fl_line( x1, y1, x2, y2 ); // diagonal line
        fl_color( orig_color );
    }

    void update_x2y2( int x_coord, int y_coord )
    {
        x2 = x_coord;
        y2 = y_coord;
    }
};//class CPrimitiveLine

class CDrawBox : public Fl_Box {
    Fl_Color border_color;
    int push_x_coord;
    int push_y_coord;
    std::vector< CPrimitiveLine > vect_primitives;
    static const int DRAW_ILLEGAL_COORDINATE = -1;

public:
    CDrawBox( int X, int Y, int W, int H )
        :
        Fl_Box( X, Y, W, H ),
        border_color( FL_BLUE ),
        push_x_coord( DRAW_ILLEGAL_COORDINATE ),
        push_y_coord( DRAW_ILLEGAL_COORDINATE )

    {
    }

    void draw() {
        // std::cout << "CDrawBox::draw()\n";
        // Make sure Fl_Box is drawn:
        Fl_Box::draw();
        draw_clear();
        draw_border();
        draw_primitives();
    }

    void draw_primitives()
    {
        for ( size_t idx = 0; idx < vect_primitives.size(); ++idx ) {
            vect_primitives[ idx ].draw();
        }
    }

    void draw_border()
    {
        Fl_Color orig_color = fl_color();
        fl_color( border_color );
        // A---B
        // |   |
        // C---D
        fl_line( x(), y(),             x() + w() - 1, y()           ); // A-B
        fl_line( x() + w() - 1, y(),   x() + w() - 1, y() + h() - 1 ); // B-D
        fl_line( x(), y() + h() - 1,   x() + w() - 1, y() + h() - 1 ); // C-D
        fl_line( x(), y(),             x(), y() + h() - 1           ); // A-C
        fl_color( orig_color );
    }

    void draw_clear()
    {
        Fl_Color orig_color = fl_color();
        fl_color( FL_BACKGROUND_COLOR ); // the default background color
        // A---B
        // |   |
        // C---D
        fl_rectf( x(), y(), x() + w() - 1, y() + h() - 1 ); // filled rectangle
        fl_color( orig_color );
    }

    int handle( int event_ )
    {
        //if ( event_ != FL_MOVE ) std::cout << "CDrawBox::handle() " << fl_eventnames[ event_ ] << " " << vect_primitives.size() << "\n";
        const int HANDLE_EVENT_HERE = 1; // indicates that we want to follow-up on this event
        const int HANDLE_EVENT_ELSEWHERE = 0; // indicates that we want don't care about this event
        int button_code = 0; // mouse button id
        int x_coord = 0;     // X coord
        int y_coord = 0;     // Y coord
        int result = Fl_Box::handle( event_ );

        switch ( event_ ) {
        case FL_PUSH: // mouse button click
            button_code = Fl::event_button(); // FL_LEFT_MOUSE FL_MIDDLE_MOUSE FL_RIGHT_MOUSE
            push_x_coord = Fl::event_x();
            push_y_coord = Fl::event_y();
            result = HANDLE_EVENT_HERE; // we want to process mouse events
            break;
        case FL_DRAG:
            // call int Fl::event_state() to find out whether ALT/SHIFT/CTRL etc. are pressed
            x_coord = Fl::event_x();
            y_coord = Fl::event_y();
            if ( push_x_coord != DRAW_ILLEGAL_COORDINATE ) {
                // first drag event since FL_PUSH
                vect_primitives.push_back( CPrimitiveLine( push_x_coord, push_y_coord, x_coord, y_coord, FL_BLACK ) );
                push_x_coord = DRAW_ILLEGAL_COORDINATE;
                push_y_coord = DRAW_ILLEGAL_COORDINATE;
            } else {
                // otherwise reposition the second end of the line segment
                assert( vect_primitives.size() );
                vect_primitives[ vect_primitives.size() - 1 ].update_x2y2( x_coord, y_coord );
            }
            result = HANDLE_EVENT_HERE; // we want to process mouse events
            Fl_Box::redraw();
            break;
        case FL_RELEASE:
            // the drag is over
            // make sure we reset our coordinates:
            push_x_coord = DRAW_ILLEGAL_COORDINATE;
            push_y_coord = DRAW_ILLEGAL_COORDINATE;
            break;
        case FL_ENTER:
            // mouse entered the box
            border_color = FL_RED;
            Fl_Box::redraw();
            break;
        case FL_LEAVE:
            // mouse left the box
            border_color = FL_BLUE;
            Fl_Box::redraw();
            break;
        }
        return result;
    }

};// class CDrawBox

#endif // CDRAWBOX_H_INCLUDED
